@@ -10,7 +10,7 @@ from jaaql.utilities.options import *
 import sys
 import os
 import jaaql.documentation as documentation
-from jaaql.utilities.utils import load_config
+from jaaql.utilities.utils import load_config, get_base_url
 from jaaql.mvc.controller import JAAQLController
 from jaaql.mvc.model import JAAQLModel
 from jaaql.mvc.controller_interface import JAAQLControllerInterface
@@ -20,12 +20,6 @@ from jaaql.config_constants import *
 
 DEFAULT__mfa_label = "test"
 
-CONFIG_KEY__security = "SECURITY"
-CONFIG_KEY_SECURITY__mfa_label = "mfa_label"
-CONFIG_KEY_SECURITY__do_audit = "do_audit"
-CONFIG_KEY__swagger = "SWAGGER"
-CONFIG_KEY_SWAGGER__url = "url"
-
 WARNING__vault_key_stdin = "MAJOR SECURITY ISSUE! Passing vault key via program arguments is insecure as other progra" \
                            "ms can see the arguments. Please provide via stdin instead!"
 
@@ -57,7 +51,8 @@ def dir_non_builtins(folder):
 
 def create_app(is_gunicorn: bool = False, override_config_path: str = None, migration_db_interface=None,
                migration_project_name: str = None, migration_folder: str = None, supplied_documentation = None,
-               controllers: [JAAQLControllerInterface] = None, models: [JAAQLModelInterface] = None, **options):
+               controllers: [JAAQLControllerInterface] = None, models: [JAAQLModelInterface] = None, additional_options: List[Option] = None,
+               **options):
     if controllers is None:
         controllers = []
     if models is None:
@@ -73,7 +68,7 @@ def create_app(is_gunicorn: bool = False, override_config_path: str = None, migr
         supplied_documentation = [supplied_documentation]
 
     if len(options) == 0 and not is_gunicorn:
-        options = parse_options(sys.argv, False)
+        options = parse_options(sys.argv, False, additional_options=additional_options)
 
     if OPT_KEY__vault_key in options:
         print(WARNING__vault_key_stdin, file=sys.stderr)
@@ -108,9 +103,9 @@ def create_app(is_gunicorn: bool = False, override_config_path: str = None, migr
         print("MFA label is set to default. This isn't a security issue but adds a nice name when added to "
               "authenticator apps via QR codes. You can change in the config")
 
-    force_mfa = config[CONFIG_KEY__security][CONFIG_KEY_SECURITY__force_mfa] in ("true", "True", True)
-    do_audit = config[CONFIG_KEY__security][CONFIG_KEY_SECURITY__do_audit] in ("true", "True", True)
-    invite_only = config[CONFIG_KEY__security][CONFIG_KEY_SECURITY__invite_only] in ("true", "True", True)
+    force_mfa = config[CONFIG_KEY__security][CONFIG_KEY_SECURITY__force_mfa] in ("true", "True", True, "TRUE")
+    do_audit = config[CONFIG_KEY__security][CONFIG_KEY_SECURITY__do_audit] in ("true", "True", True, "TRUE")
+    invite_only = config[CONFIG_KEY__security][CONFIG_KEY_SECURITY__invite_only] in ("true", "True", True, "TRUE")
 
     # The following code sets it "properly" in case it is accessed incorrectly from elsewhere
     config[CONFIG_KEY__security][CONFIG_KEY_SECURITY__force_mfa] = force_mfa
@@ -124,7 +119,7 @@ def create_app(is_gunicorn: bool = False, override_config_path: str = None, migr
 
     model = JAAQLModel(config, vault_key, migration_db_interface, migration_project_name, migration_folder,
                        is_container=is_gunicorn, url=url)
-    controller = JAAQLController(model, is_gunicorn)
+    controller = JAAQLController(model, is_gunicorn, get_base_url(config, is_gunicorn))
     controller.create_app()
 
     for sub_controller in controllers:
